#!/usr/bin/perl -w

# ------------------------- #
# cisco_dhcp_stats.pl
# ------------------------- #
# 
# Description :
#   script to retrieve
#   DHCP infos on Cisco
#   routers who do not
#   support DHCP MIBs
# 
# 20101216 .Sam.
# http://www.l33.fr
# ------------------------- #

use strict;
#use warnings;
use Getopt::Long;
use Net::Telnet::Cisco;

# global vars
my ($appli,$host,$port,$username,$password,$enable,$pool,$result);

# process command line options :
# --host <hostname> --port <port> --username <username> --password <password> --enable <enable> --appli poolstats --pool <poolname>
# --host <hostname> --port <port> --username <username> --password <password> --enable <enable> --appli conflicts
# --host <hostname> --port <port> --username <username> --password <password> --enable <enable> --appli memstats

GetOptions( "host=s"    => \$host,
            "port=s"    => \$port,
            "username=s"=> \$username,
            "password=s"=> \$password,
            "enable=s"  => \$enable,
            "appli=s"   => \$appli,
            "pool:s"    => \$pool);

my $timeout = 5;

# cleaning function 
sub trim($)
{
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;

    return $string;
}

sub my_connect()
{
    # open a new session
    my $session = Net::Telnet::Cisco->new(
    Host => $host,
    Port => $port,
    Errmode => sub { die "Unable to connect to $host on port $port\n"; });

    # log in to the equipment
    $session->login(Name => $username,
                    Password => $password,
                    Timeout => $timeout); 

    return $session;
}

sub get_stats($)
{
    my $session = shift;

    # clean the term output
    $session->cmd('terminal length 0');

    # enable cisco mode
    if ($session->enable($enable))
    {
        for ($appli)
        {
            if (/poolstats/)
            {
                my ($ipe,$ipl,$ipm) = fpool($session);
                $session->close;
                $result = "poolexcl:$ipe poolleased:$ipl poolmax:$ipm ";

                return $result;
            }
            elsif (/conflicts/)
            {
                my ($arpd,$pingd) = fconflicts($session);
                $session->close;
                $result = "arpdetect:$arpd pingdetect:$pingd ";

                return $result;
            }
            elsif (/memstats/)
            {
                my ($dmem) = fdmem($session);
                $session->close;
                $result = "$dmem";

                return $result;
            }
            else
            {
                warn "ERROR: bad -a argument: " . $session->errmsg;
                $session->close;
                exit 0;
            }
        }
    }
    else
    {
        warn "ERROR: can t enable: " . $session->errmsg;
        $session->close;
        exit 0;
    }
}

sub fpool($)
{
    my $session = shift;
    my ($c,$ipe,$ipl,$ipm);
    if ($pool)
    {
        $c="show ip dhcp pool $pool";
    }
    else
    {
        $c="show ip dhcp pool";
    }
    my @stats = $session->cmd($c);

    # get the max pool addr
    my $line1 = trim($stats[4]);
    chomp($line1);
    my @tt = split(' ',$line1);
    $ipm = $tt[3];

    # get current leased addr
    my $line2 = trim($stats[5]);
    chomp($line2);
    my @tl = split(' ',$line2);
    $ipl = $tl[3];

    # get nb of excluded addr
    my $line3 = trim($stats[6]);
    chomp($line3);
    my @te = split(' ',$line3);
    if ($te[0] =~ /Excluded/)
    {
        $ipe = $te[3];
    }
    else
    {
        $ipe = '0';
    }

    return $ipe,$ipl,$ipm;
}

sub fconflicts($)
{
    my $session = shift;
    my ($arpd,$pingd);
    my @conflicts = $session->cmd('show ip dhcp conflict');
    my $line = trim(@conflicts);
    chomp($line);
    my @cf = split(' ',$line);
    my $i=0;
    {
        foreach (@cf)
        {
            if (/ARP/) {
                $i++;
                $arpd = $i;
            } else {
                $arpd = 0;
            }
            if (/Ping/) {
                $i++;
                $pingd = $i;
            } else {
                $pingd = 0;
            }
        }
    }

    return ($arpd,$pingd);
}

sub fdmem($)
{
    my $session = shift;
    my ($dmem);
    my @serv = $session->cmd('show ip dhcp server statistics');
    my $line = trim($serv[0]);
    chomp($line);
    my @dm = split(' ',$line);
    $dmem = $dm[2];

    return ($dmem);
}

my $s = my_connect();
get_stats($s);
print $result;
